@@ -55,7 +55,6 @@ namespace core {
 
 namespace display {
   class Manager;
-  class ClientInfo;
 }
 
 namespace input {
@@ -98,8 +97,6 @@ public:
   CommandScheduler*   command_scheduler()           { return m_commandScheduler; }
   utils::VariableMap* variable()                    { return m_variables; }
 
-  display::ClientInfo* client_info()                { return m_clientInfo; }
-
   uint64_t            tick() const                  { return m_tick; }
   void                inc_tick()                    { m_tick++; }
 
@@ -128,8 +125,6 @@ private:
   CommandScheduler*   m_commandScheduler;
   utils::VariableMap* m_variables;
 
-  display::ClientInfo* m_clientInfo;
-
   uint64_t            m_tick;
 
   mode_t              m_umask;
