@@ -46,6 +46,7 @@
 #include <torrent/torrent.h>
 #include <torrent/tracker.h>
 #include <torrent/tracker_list.h>
+#include <torrent/data/file_list.h>
 
 #include "utils/variable_generic.h"
 
@@ -55,7 +56,6 @@ namespace core {
 
 Download::Download(download_type d) :
   m_download(d),
-  m_fileList(d.file_list()),
   m_trackerList(d.tracker_list()),
 
   m_hashFailed(false),
@@ -90,18 +90,24 @@ Download::Download(download_type d) :
   // resume/pause.
   m_variables.insert("state_changed",      new utils::VariableObject(bencode(), "rtorrent", "state_changed", torrent::Object::TYPE_VALUE));
 
-  m_variables.insert("directory",          new utils::VariableStringSlot(rak::mem_fn(&m_fileList, &torrent::FileList::root_dir), rak::mem_fn(this, &Download::set_root_directory)));
+  m_variables.insert("directory",          new utils::VariableStringSlot(rak::mem_fn(m_download.file_list(), &torrent::FileList::root_dir), rak::mem_fn(this, &Download::set_root_directory)));
 
-  m_variables.insert("info_hash",          new utils::VariableStringSlot(rak::mem_fn(&m_download, &torrent::Download::info_hash), NULL));
+//   m_variables.insert("info_hash",          new utils::VariableStringSlot(rak::mem_fn(&m_download, &torrent::Download::info_hash), NULL));
 
   m_variables.insert("min_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_min), rak::mem_fn(&m_download, &download_type::set_peers_min)));
   m_variables.insert("max_peers",          new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::peers_max), rak::mem_fn(&m_download, &download_type::set_peers_max)));
   m_variables.insert("max_uploads",        new utils::VariableValueSlot(rak::mem_fn(&m_download, &download_type::uploads_max), rak::mem_fn(&m_download, &download_type::set_uploads_max)));
 
+  m_variables.insert("max_file_size",      new utils::VariableValueSlot(rak::mem_fn(file_list(), &file_list_type::max_file_size), rak::mem_fn(file_list(), &file_list_type::set_max_file_size)));
+//   m_variables.insert("split_file_size",    new utils::VariableValueSlot(rak::mem_fn(file_list(), &file_list_type::split_file_size), rak::mem_fn(file_list(), &file_list_type::set_split_file_size)));
+//   m_variables.insert("split_suffix",       new utils::VariableStringSlot(rak::mem_fn(file_list(), &file_list_type::split_suffix), rak::mem_fn(file_list(), &file_list_type::set_split_suffix)));
+
   m_variables.insert("up_rate",            new utils::VariableValueSlot(rak::mem_fn(m_download.up_rate(), &torrent::Rate::rate), NULL));
   m_variables.insert("up_total",           new utils::VariableValueSlot(rak::mem_fn(m_download.up_rate(), &torrent::Rate::total), NULL));
   m_variables.insert("down_rate",          new utils::VariableValueSlot(rak::mem_fn(m_download.down_rate(), &torrent::Rate::rate), NULL));
   m_variables.insert("down_total",         new utils::VariableValueSlot(rak::mem_fn(m_download.down_rate(), &torrent::Rate::total), NULL));
+  m_variables.insert("skip_rate",          new utils::VariableValueSlot(rak::mem_fn(m_download.skip_rate(), &torrent::Rate::rate), NULL));
+  m_variables.insert("skip_total",         new utils::VariableValueSlot(rak::mem_fn(m_download.skip_rate(), &torrent::Rate::total), NULL));
 
   m_variables.insert("priority",           new utils::VariableValueSlot(rak::mem_fn(this, &Download::priority), rak::mem_fn(this, &Download::set_priority)));
 
@@ -221,7 +227,7 @@ Download::priority_to_string(uint32_t p) {
 float
 Download::distributed_copies() const {
   const uint8_t* avail = m_download.chunks_seen();
-  const uint8_t* end = avail + m_download.chunks_total();
+  const uint8_t* end = avail + m_download.file_list()->size_chunks();
 
   if (avail == NULL)
     return 0;
@@ -237,7 +243,7 @@ Download::distributed_copies() const {
       num = 1;
     }
 
-  return minAvail + 1 - (float)num / m_download.chunks_total();
+  return minAvail + 1 - (float)num / m_download.file_list()->size_chunks();
 }
 
 void
@@ -248,15 +254,17 @@ Download::receive_chunk_failed(__UNUSED uint32_t idx) {
 // Clean up.
 void
 Download::set_root_directory(const std::string& path) {
+  torrent::FileList* fileList = m_download.file_list();
+
   if (path.empty()) {
-    m_fileList.set_root_dir("./" + (m_fileList.size() > 1 ? m_download.name() : std::string()));
+    fileList->set_root_dir("./" + (fileList->size_files() > 1 ? m_download.name() : std::string()));
 
   } else {
     std::string fullPath = rak::path_expand(path);
 
-    m_fileList.set_root_dir(fullPath +
-			    (*fullPath.rbegin() != '/' ? "/" : "") +
-			    (m_fileList.size() > 1 ? m_download.name() : ""));
+    fileList->set_root_dir(fullPath +
+                           (*fullPath.rbegin() != '/' ? "/" : "") +
+                           (fileList->size_files() > 1 ? m_download.name() : ""));
   }
 
   bencode()->get_key("rtorrent").insert_key("directory", path);
