@@ -37,15 +37,18 @@
 #ifndef RTORRENT_CORE_DOWNLOAD_H
 #define RTORRENT_CORE_DOWNLOAD_H
 
-#include <rak/string_manip.h>
 #include <sigc++/connection.h>
 #include <torrent/download.h>
-#include <torrent/file_list.h>
+#include <torrent/hash_string.h>
 #include <torrent/tracker_list.h>
-#include <torrent/torrent.h>
+#include <torrent/data/file_list.h>
 
 #include "utils/variable_map.h"
 
+namespace torrent {
+  class TrackerList;
+}
+
 namespace core {
 
 class Download {
@@ -66,7 +69,7 @@ public:
 
   bool                is_open() const                          { return m_download.is_open(); }
   bool                is_active() const                        { return m_download.is_active(); }
-  bool                is_done() const                          { return m_download.chunks_done() == m_download.chunks_total(); }
+  bool                is_done() const                          { return m_download.file_list()->is_done(); }
   bool                is_downloading() const                   { return is_active() && !is_done(); }
   bool                is_seeding() const                       { return is_active() && is_done(); }
 
@@ -86,8 +89,10 @@ public:
   download_type*       download()                              { return &m_download; }
   const download_type* c_download() const                      { return &m_download; }
 
+  file_list_type*       file_list()                            { return m_download.file_list(); }
+  const file_list_type* c_file_list() const                    { return m_download.file_list(); }
+
   torrent::Object*    bencode()                                { return m_download.bencode(); }
-  file_list_type*     file_list()                              { return &m_fileList; }
   tracker_list_type*  tracker_list()                           { return &m_trackerList; }
 
   const std::string&  message() const                          { return m_message; }
@@ -103,12 +108,12 @@ public:
   // Helper functions for calling functions in download_type
   // through sigc++.
   template <typename Ret, Ret (download_type::*func)()>
-  void                call()                                                { (m_download.*func)(); }
+  void                call()                                               { (m_download.*func)(); }
 
   template <typename Ret, typename Arg1, Ret (download_type::*func)(Arg1)>
-  void                call(Arg1 a1)                                         { (m_download.*func)(a1); }
+  void                call(Arg1 a1)                                        { (m_download.*func)(a1); }
 
-  bool operator == (const std::string& str)                                { return str == m_download.info_hash(); }
+  bool                operator == (const std::string& str) const;
 
   void                set_connection_type(const std::string& t) { m_download.set_connection_type(string_to_connection_type(t)); }
 
@@ -136,7 +141,6 @@ private:
 
   // Store the FileList instance so we can use slots etc on it.
   download_type       m_download;
-  file_list_type      m_fileList;
   tracker_list_type   m_trackerList;
 
   bool                m_hashFailed;
@@ -151,6 +155,11 @@ private:
   sigc::connection    m_connStorageError;
 };
 
+inline bool
+Download::operator == (const std::string& str) const {
+  return str.size() == torrent::HashString::size_data && *torrent::HashString::cast_from(str) == m_download.info_hash();
+}
+
 }
 
 #endif
