@@ -45,6 +45,7 @@
 #include <torrent/exceptions.h>
 #include <torrent/rate.h>
 #include <torrent/resume.h>
+#include <torrent/data/file_utils.h>
 
 #include "utils/variable_generic.h"
 
@@ -100,7 +101,7 @@ DownloadFactory::commit() {
 void
 DownloadFactory::receive_load() {
   if (m_stream)
-    throw torrent::client_error("DownloadFactory::load() called on an object with m_stream != NULL");
+    throw torrent::internal_error("DownloadFactory::load() called on an object with m_stream != NULL");
 
   if (std::strncmp(m_uri.c_str(), "http://", 7) == 0) {
     // Http handling here.
@@ -142,7 +143,7 @@ DownloadFactory::receive_commit() {
 void
 DownloadFactory::receive_success() {
   if (m_stream == NULL)
-    throw torrent::client_error("DownloadFactory::receive_success() called on an object with m_stream == NULL.");
+    throw torrent::internal_error("DownloadFactory::receive_success() called on an object with m_stream == NULL.");
 
   Download* download = m_manager->download_list()->create(m_stream, m_printLog);
 
@@ -157,7 +158,8 @@ DownloadFactory::receive_success() {
 
   if (!m_session) {
     // We only allow session torrents to keep their
-    // 'rtorrent/libtorrent' sections.
+    // 'rtorrent/libtorrent' sections. The "fast_resume" section
+    // should be safe to keep.
     root->erase_key("rtorrent");
     root->erase_key("libtorrent");
   }
@@ -189,6 +191,18 @@ DownloadFactory::receive_success() {
   if (!control->variable()->get_value("use_udp_trackers"))
     download->enable_udp_trackers(false);
 
+  if (control->variable()->get_value("max_file_size") > 0)
+    download->variable()->set("max_file_size", control->variable()->get("max_file_size"));
+
+  // Check first if we already have these values set in the session
+  // torrent, so that it is safe to change the values.
+  //
+  // Need to also catch the exceptions.
+  if (control->variable()->get_value("split_file_size") >= 0)
+    torrent::file_split_all(download->download()->file_list(),
+                            control->variable()->get_value("split_file_size"),
+                            control->variable()->get_string("split_suffix"));
+
   if (!rtorrent->has_key_string("directory"))
     download->variable()->set("directory", m_variables.get("directory"));
   else
@@ -238,7 +252,7 @@ DownloadFactory::receive_success() {
 void
 DownloadFactory::receive_failed(const std::string& msg) {
   if (m_stream == NULL)
-    throw torrent::client_error("DownloadFactory::receive_success() called on an object with m_stream == NULL");
+    throw torrent::internal_error("DownloadFactory::receive_success() called on an object with m_stream == NULL");
 
   // Add message to log.
   if (m_printLog) {
@@ -286,7 +300,8 @@ DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorre
     download->download()->up_rate()->set_total(rtorrent->get_key_value("total_uploaded"));
 
   if (rtorrent->has_key_value("chunks_done"))
-    download->download()->set_chunks_done(std::min<uint32_t>(rtorrent->get_key_value("chunks_done"), download->download()->chunks_total()));
+    download->download()->set_chunks_done(std::min<uint32_t>(rtorrent->get_key_value("chunks_done"),
+                                                             download->download()->file_list()->size_chunks()));
 
   if (!rtorrent->has_key_value("ignore_commands"))
     rtorrent->insert_key("ignore_commands", (int64_t)0);
