@@ -51,6 +51,7 @@
 #include <sigc++/hide.h>
 #include <torrent/object.h>
 #include <torrent/connection_manager.h>
+#include <torrent/error.h>
 #include <torrent/exceptions.h>
 #include <torrent/resume.h>
 #include <torrent/tracker_list.h>
@@ -105,6 +106,65 @@ receive_tracker_dump(const std::string& url, const char* data, size_t size) {
   fstr << std::endl <<"---" << std::endl;
 }
 
+void
+Manager::handshake_log(const sockaddr* sa, int msg, int err, const torrent::HashString* hash) {
+  if (!control->variable()->get_value("handshake_log"))
+    return;
+  
+  std::string peer;
+  std::string download;
+
+  const rak::socket_address* socketAddress = rak::socket_address::cast_from(sa);
+
+  if (socketAddress->is_valid()) {
+    char port[6];
+    snprintf(port, sizeof(port), "%d", socketAddress->port());
+    peer = socketAddress->address_str() + ":" + port;
+  } else {
+    peer = "(unknown)";
+  }
+
+//   torrent::Download d = torrent::download_find(hash);
+
+//   if (d.is_valid())
+//     download = ": " + d.name();
+//   else
+    download = "";
+
+  switch (msg) {
+  case torrent::ConnectionManager::handshake_incoming:
+    m_logComplete.push_front("Incoming connection from " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_outgoing:
+    m_logComplete.push_front("Outgoing connection to " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_outgoing_encrypted:
+    m_logComplete.push_front("Outgoing encrypted connection to " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_outgoing_proxy:
+    m_logComplete.push_front("Outgoing proxy connection to " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_success:
+    m_logComplete.push_front("Successful handshake: " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_dropped:
+    m_logComplete.push_front("Dropped handshake: " + peer + " - " + torrent::strerror(err) + download);
+    break;
+  case torrent::ConnectionManager::handshake_failed:
+    m_logComplete.push_front("Handshake failed: " + peer + " - " + torrent::strerror(err) + download);
+    break;
+  case torrent::ConnectionManager::handshake_retry_plaintext:
+    m_logComplete.push_front("Trying again without encryption: " + peer + download);
+    break;
+  case torrent::ConnectionManager::handshake_retry_encrypted:
+    m_logComplete.push_front("Trying again encrypted: " + peer + download);
+    break;
+  default:
+    m_logComplete.push_front("Unknown handshake message for " + peer + download);
+    break;
+  }
+}
+
 // Hmm... find some better place for all this.
 void
 Manager::delete_tied(Download* d) {
@@ -144,7 +204,7 @@ Manager::~Manager() {
 void
 Manager::set_hashing_view(View* v) {
   if (v == NULL || m_hashingView != NULL)
-    throw torrent::client_error("Manager::set_hashing_view(...) received NULL or is already set.");
+    throw torrent::internal_error("Manager::set_hashing_view(...) received NULL or is already set.");
 
   m_hashingView = v;
   v->signal_changed().connect(sigc::mem_fun(this, &Manager::receive_hashing_changed));
@@ -183,6 +243,8 @@ Manager::initialize_second() {
   m_downloadList->slot_map_insert()["1_connect_tracker_dump"] = sigc::bind(sigc::ptr_fun(&connect_signal_tracker_dump), sigc::ptr_fun(&receive_tracker_dump));
 
   m_downloadList->slot_map_erase()["1_delete_tied"] = sigc::mem_fun(this, &Manager::delete_tied);
+
+  torrent::connection_manager()->set_signal_handshake_log(sigc::mem_fun(this, &Manager::handshake_log));
 }
 
 void
@@ -293,6 +355,42 @@ Manager::set_local_address(const std::string& addr) {
   }
 }
 
+std::string
+Manager::proxy_address() const {
+  return rak::socket_address::cast_from(torrent::connection_manager()->proxy_address())->address_str();
+}
+
+void
+Manager::set_proxy_address(const std::string& addr) {
+  int port;
+  rak::address_info* ai;
+
+  char buf[addr.length() + 1];
+
+  int err = std::sscanf(addr.c_str(), "%[^:]:%i", buf, &port);
+
+  if (err <= 0)
+    throw torrent::input_error("Could not parse proxy address.");
+
+  if (err == 1)
+    port = 80;
+
+  if ((err = rak::address_info::get_address_info(buf, PF_INET, SOCK_STREAM, &ai)) != 0)
+    throw torrent::input_error("Could not set proxy address: " + std::string(rak::address_info::strerror(err)) + ".");
+  
+  try {
+
+    ai->address()->set_port(port);
+    torrent::connection_manager()->set_proxy_address(ai->address()->c_sockaddr());
+    
+    rak::address_info::free_address_info(ai);
+
+  } catch (torrent::input_error& e) {
+    rak::address_info::free_address_info(ai);
+    throw e;
+  }
+}
+
 void
 Manager::receive_http_failed(std::string msg) {
   m_logImportant.push_front("Http download error: \"" + msg + "\"");
@@ -403,9 +501,11 @@ Manager::receive_hashing_changed() {
       }
 
       if ((*itr)->is_hash_checked())
-        throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
+        throw torrent::internal_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
   
-      if ((*itr)->is_hash_failed() || (*itr)->variable()->get_value("hashing") != Download::variable_hashing_initial)
+      if ((*itr)->is_hash_failed() ||
+          (*itr)->variable()->get_value("hashing") != Download::variable_hashing_initial ||
+          (*itr)->download()->chunks_hashed() != 0)
         continue;
 
       m_downloadList->open_throw(*itr);
@@ -435,7 +535,7 @@ Manager::receive_hashing_changed() {
     try {
 
       if ((*itr)->is_hash_checked() || (*itr)->is_hash_checking())
-        throw torrent::client_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
+        throw torrent::internal_error("core::Manager::receive_hashing_changed() hash already checked or checking.");
   
       if ((*itr)->is_hash_failed())
         continue;
