@@ -162,7 +162,7 @@ ViewManager::clear() {
 ViewManager::iterator
 ViewManager::insert(const std::string& name) {
   if (find(name) != end())
-    throw torrent::client_error("ViewManager::insert(...) name already inserted.");
+    throw torrent::internal_error("ViewManager::insert(...) name already inserted.");
 
   View* view = new View();
   view->initialize(name, m_list);
