@@ -44,8 +44,11 @@
 #include <torrent/exceptions.h>
 #include <torrent/connection_manager.h>
 #include <torrent/rate.h>
+#include <torrent/torrent.h>
 #include <torrent/tracker.h>
 #include <torrent/tracker_list.h>
+#include <torrent/data/file_list.h>
+#include <torrent/peer/client_info.h>
 
 #include "core/download.h"
 
@@ -79,7 +82,7 @@ print_hhmmss_local(char* first, char* last, time_t t) {
   
   if (u == NULL)
     //return "inv_time";
-    throw torrent::client_error("print_hhmmss_local(...) failed.");
+    throw torrent::internal_error("print_hhmmss_local(...) failed.");
 
   return print_buffer(first, last, "%2u:%02u:%02u", u->tm_hour, u->tm_min, u->tm_sec);
 }
@@ -98,7 +101,7 @@ print_ddmmyyyy(char* first, char* last, time_t t) {
   
   if (u == NULL)
     //return "inv_time";
-    throw torrent::client_error("print_ddmmyyyy(...) failed.");
+    throw torrent::internal_error("print_ddmmyyyy(...) failed.");
 
   return print_buffer(first, last, "%02u/%02u/%04u", u->tm_mday, (u->tm_mon + 1), (1900 + u->tm_year));
 }
@@ -131,11 +134,11 @@ print_download_info(char* first, char* last, core::Download* d) {
     first = print_buffer(first, last, "          ");
 
   if (d->is_done())
-    first = print_buffer(first, last, "done %10.1f MB", (double)d->download()->bytes_total() / (double)(1 << 20));
+    first = print_buffer(first, last, "done %10.1f MB", (double)d->download()->file_list()->size_bytes() / (double)(1 << 20));
   else
     first = print_buffer(first, last, "%6.1f / %6.1f MB",
                          (double)d->download()->bytes_done() / (double)(1 << 20),
-                         (double)d->download()->bytes_total() / (double)(1 << 20));
+                         (double)d->download()->file_list()->size_bytes() / (double)(1 << 20));
   
   first = print_buffer(first, last, " Rate: %5.1f / %5.1f KB Uploaded: %7.1f MB",
                        (double)d->download()->up_rate()->rate() / (1 << 10),
@@ -163,7 +166,7 @@ print_download_info(char* first, char* last, core::Download* d) {
     first = print_buffer(first, last, "]");
 
   if (first > last)
-    throw torrent::client_error("print_download_info(...) wrote past end of the buffer.");
+    throw torrent::internal_error("print_download_info(...) wrote past end of the buffer.");
 
   return first;
 }
@@ -179,7 +182,7 @@ print_download_status(char* first, char* last, core::Download* d) {
 
   if (d->is_hash_checking()) {
     first = print_buffer(first, last, "Checking hash [%2i%%]",
-                         (d->download()->chunks_hashed() * 100) / d->download()->chunks_total());
+                         (d->download()->chunks_hashed() * 100) / d->download()->file_list()->size_chunks());
 
   } else if (d->tracker_list()->is_busy() && d->tracker_list()->focus() < d->tracker_list()->size()) {
     torrent::TrackerList* tl = d->tracker_list();
@@ -195,7 +198,7 @@ print_download_status(char* first, char* last, core::Download* d) {
   }
 
   if (first > last)
-    throw torrent::client_error("print_download_status(...) wrote past end of the buffer.");
+    throw torrent::internal_error("print_download_status(...) wrote past end of the buffer.");
 
   return first;
 }
@@ -207,7 +210,7 @@ print_download_time_left(char* first, char* last, core::Download* d) {
   if (rate < 512)
     return print_buffer(first, last, "--d --:--");
   
-  time_t remaining = (d->download()->bytes_total() - d->download()->bytes_done()) / (rate & ~(uint32_t)(512 - 1));
+  time_t remaining = (d->download()->file_list()->size_bytes() - d->download()->bytes_done()) / (rate & ~(uint32_t)(512 - 1));
 
   return print_ddhhmm(first, last, remaining);
 }
@@ -218,7 +221,25 @@ print_download_percentage_done(char* first, char* last, core::Download* d) {
     //return print_buffer(first, last, "[--%%]");
     return print_buffer(first, last, "     ");
   else
-    return print_buffer(first, last, "[%2u%%]", (d->download()->chunks_done() * 100) / d->download()->chunks_total());
+    return print_buffer(first, last, "[%2u%%]", (d->download()->file_list()->completed_chunks() * 100) / d->download()->file_list()->size_chunks());
+}
+
+char*
+print_client_version(char* first, char* last, const torrent::ClientInfo& clientInfo) {
+  switch (torrent::ClientInfo::version_size(clientInfo.type())) {
+  case 4:
+    return print_buffer(first, last, "%s %hhu.%hhu.%hhu.%hhu",
+                        clientInfo.short_description(),
+                        clientInfo.version()[0], clientInfo.version()[1],
+                        clientInfo.version()[2], clientInfo.version()[3]);
+  case 3:
+    return print_buffer(first, last, "%s %hhu.%hhu.%hhu",
+                        clientInfo.short_description(),
+                        clientInfo.version()[0], clientInfo.version()[1],
+                        clientInfo.version()[2]);
+  default:
+    return print_buffer(first, last, "%s", clientInfo.short_description());
+  }
 }
 
 char*
@@ -246,7 +267,7 @@ print_status_info(char* first, char* last) {
   }
   
   if (first > last)
-    throw torrent::client_error("print_status_info(...) wrote past end of the buffer.");
+    throw torrent::internal_error("print_status_info(...) wrote past end of the buffer.");
 
   if (!rak::socket_address::cast_from(torrent::connection_manager()->bind_address())->is_address_any()) {
     first = print_buffer(first, last, " [Bind ");
@@ -265,8 +286,8 @@ print_status_extra(char* first, char* last, __UNUSED Control* c) {
 
   first = print_buffer(first, last, " [S %i/%i/%i]",
                        torrent::total_handshakes(),
-                       torrent::open_sockets(),
-                       torrent::max_open_sockets());
+                       torrent::connection_manager()->size(),
+                       torrent::connection_manager()->max_size());
                        
   first = print_buffer(first, last, " [F %i/%i]",
                        torrent::open_files(),
