@@ -39,8 +39,8 @@
 #include <cmath>
 #include <stdexcept>
 #include <rak/string_manip.h>
-#include <torrent/block_list.h>
-#include <torrent/transfer_list.h>
+#include <torrent/data/block_list.h>
+#include <torrent/data/transfer_list.h>
 
 #include "core/download.h"
 
