@@ -43,6 +43,7 @@
 
 namespace torrent {
   class File;
+  class FileListIterator;
 }
 
 namespace core {
@@ -53,18 +54,17 @@ namespace display {
 
 class WindowFileList : public Window {
 public:
-  typedef std::pair<unsigned int, unsigned int> Range;
+  typedef torrent::FileListIterator iterator;
 
-  WindowFileList(core::Download* d, unsigned int* focus);
+  WindowFileList(core::Download* d, iterator* selected);
 
-  virtual void     redraw();
+  virtual void        redraw();
 
 private:
-  int              done_percentage(torrent::File& e);
+  int                 done_percentage(torrent::File* e);
 
-  core::Download*  m_download;
-
-  unsigned int*    m_focus;
+  core::Download*     m_download;
+  iterator*           m_selected;
 };
 
 }
