@@ -49,12 +49,13 @@
 #include <torrent/chunk_manager.h>
 #include <torrent/connection_manager.h>
 #include <torrent/exceptions.h>
-#include <torrent/file.h>
 #include <torrent/path.h>
 #include <torrent/rate.h>
 #include <torrent/torrent.h>
 #include <torrent/tracker.h>
 #include <torrent/tracker_list.h>
+#include <torrent/data/file.h>
+#include <torrent/data/file_list.h>
 
 #include "core/download.h"
 #include "core/download_list.h"
@@ -175,17 +176,11 @@ apply_close_low_diskspace(Control* m, int64_t arg) {
   core::Manager::DListItr itr = m->core()->download_list()->begin();
 
   while ((itr = std::find_if(itr, m->core()->download_list()->end(), std::mem_fun(&core::Download::is_downloading))) != m->core()->download_list()->end()) {
-    rak::fs_stat stat;
-    std::string path = (*itr)->file_list()->root_dir() + (*itr)->file_list()->get(0).path()->as_string();
-
-    if (!stat.update(path)) {
-      m->core()->push_log(std::string("Cannot read free diskspace: ") + strerror(errno) + " for " + path);
-
-    } else if (stat.bytes_avail() < arg) {
+    if ((*itr)->file_list()->free_diskspace() < (uint64_t)arg) {
       m->core()->download_list()->close(*itr);
 
       (*itr)->set_hash_failed(true);
-      (*itr)->set_message(std::string("Low diskspace"));
+      (*itr)->set_message(std::string("Low diskspace."));
     }
 
     ++itr;
@@ -228,6 +223,36 @@ apply_encoding_list(__UNUSED Control* m, const std::string& arg) {
   torrent::encoding_list()->push_back(arg);
 }
 
+void
+apply_encryption(const std::string& arg) {
+  rak::split_iterator_t<std::string> sitr = rak::split_iterator(arg, ',');
+  uint32_t options_mask = torrent::ConnectionManager::encryption_none;
+
+  while (sitr != rak::split_iterator(arg)) {
+    std::string opt = rak::trim(*sitr);
+    ++sitr;
+
+    if (opt == "none")
+      options_mask = torrent::ConnectionManager::encryption_none;
+    else if (opt == "allow_incoming")
+      options_mask |= torrent::ConnectionManager::encryption_allow_incoming;
+    else if (opt == "try_outgoing")
+      options_mask |= torrent::ConnectionManager::encryption_try_outgoing;
+    else if (opt == "require")
+      options_mask |= torrent::ConnectionManager::encryption_require;
+    else if (opt == "require_RC4" || opt == "require_rc4")
+      options_mask |= torrent::ConnectionManager::encryption_require_RC4;
+    else if (opt == "enable_retry")
+      options_mask |= torrent::ConnectionManager::encryption_enable_retry;
+    else if (opt == "prefer_plaintext")
+      options_mask |= torrent::ConnectionManager::encryption_prefer_plaintext;
+    else
+      throw torrent::input_error("Invalid encryption option '" + opt + "'.");
+  }
+
+  torrent::connection_manager()->set_encryption_options(options_mask);
+}
+
 void
 apply_enable_trackers(Control* m, __UNUSED const std::string& arg) {
   bool state = (arg != "no");
@@ -249,25 +274,28 @@ apply_enable_trackers(Control* m, __UNUSED const std::string& arg) {
 
 void
 apply_tos(const std::string& arg) {
+  utils::Variable::value_type value;
   torrent::ConnectionManager* cm = torrent::connection_manager();
 
   if (arg == "default")
-    cm->set_priority(torrent::ConnectionManager::iptos_default);
+    value = torrent::ConnectionManager::iptos_default;
 
   else if (arg == "lowdelay")
-    cm->set_priority(torrent::ConnectionManager::iptos_lowdelay);
+    value = torrent::ConnectionManager::iptos_lowdelay;
 
   else if (arg == "throughput")
-    cm->set_priority(torrent::ConnectionManager::iptos_throughput);
+    value = torrent::ConnectionManager::iptos_throughput;
 
   else if (arg == "reliability")
-    cm->set_priority(torrent::ConnectionManager::iptos_reliability);
+    value = torrent::ConnectionManager::iptos_reliability;
 
   else if (arg == "mincost")
-    cm->set_priority(torrent::ConnectionManager::iptos_mincost);
+    value = torrent::ConnectionManager::iptos_mincost;
 
-  else 
+  else if (!utils::Variable::string_to_value_unit_nothrow(arg.c_str(), &value, 16, 0))
     throw torrent::input_error("Invalid TOS identifier.");
+
+  cm->set_priority(value);
 }
 
 void
@@ -419,6 +447,8 @@ initialize_option_handler(Control* c) {
                                                                            rak::mem_fn(control->core(), &core::Manager::set_bind_address)));
   variables->insert("ip",                    new utils::VariableStringSlot(rak::mem_fn(control->core(), &core::Manager::local_address),
                                                                            rak::mem_fn(control->core(), &core::Manager::set_local_address)));
+  variables->insert("connection_proxy",      new utils::VariableStringSlot(rak::mem_fn(control->core(), &core::Manager::proxy_address),
+                                                                           rak::mem_fn(control->core(), &core::Manager::set_proxy_address)));
 
   variables->insert("http_proxy",            new utils::VariableStringSlot(rak::mem_fn(c->core()->get_poll_manager()->get_http_stack(), &core::CurlStack::http_proxy),
                                                                            rak::mem_fn(c->core()->get_poll_manager()->get_http_stack(), &core::CurlStack::set_http_proxy)));
@@ -439,7 +469,8 @@ initialize_option_handler(Control* c) {
 
   variables->insert("hash_max_tries",        new utils::VariableValueSlot(rak::ptr_fn(&torrent::hash_max_tries), rak::ptr_fn(&torrent::set_hash_max_tries)));
   variables->insert("max_open_files",        new utils::VariableValueSlot(rak::ptr_fn(&torrent::max_open_files), rak::ptr_fn(&torrent::set_max_open_files)));
-  variables->insert("max_open_sockets",      new utils::VariableValueSlot(rak::ptr_fn(&torrent::max_open_sockets), rak::ptr_fn(&torrent::set_max_open_sockets)));
+  variables->insert("max_open_sockets",      new utils::VariableValueSlot(rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::max_size),
+                                                                          rak::mem_fn(torrent::connection_manager(), &torrent::ConnectionManager::set_max_size)));
 
   variables->insert("print",                 new utils::VariableStringSlot(rak::value_fn(std::string()), rak::mem_fn(control->core(), &core::Manager::push_log)));
   variables->insert("import",                new utils::VariableStringSlot(rak::value_fn(std::string()), rak::ptr_fn(&apply_import)));
@@ -479,6 +510,20 @@ initialize_option_handler(Control* c) {
   variables->insert("timeout_safe_sync",     new utils::VariableValueSlot(rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::timeout_safe_sync),
                                                                           rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::set_timeout_safe_sync)));
 
+  variables->insert("preload_type",          new utils::VariableValueSlot(rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::preload_type),
+                                                                          rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::set_preload_type)));
+
+  variables->insert("preload_min_size",      new utils::VariableValueSlot(rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::preload_min_size),
+                                                                          rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::set_preload_min_size)));
+
+  variables->insert("preload_required_rate", new utils::VariableValueSlot(rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::preload_required_rate),
+                                                                          rak::mem_fn(torrent::chunk_manager(), &torrent::ChunkManager::set_preload_required_rate),
+                                                                          0, (1 << 10)));
+
+  variables->insert("max_file_size",         new utils::VariableValue(-1));
+  variables->insert("split_file_size",       new utils::VariableValue(-1));
+  variables->insert("split_suffix",          new utils::VariableAny(".part"));
+
   variables->insert("port_range",            new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_port_range, c)));
 
   variables->insert("hash_read_ahead",       new utils::VariableValueSlot(rak::ptr_fn(torrent::hash_read_ahead), rak::bind_ptr_fn(&apply_hash_read_ahead, c)));
@@ -503,4 +548,7 @@ initialize_option_handler(Control* c) {
 
   variables->insert("enable_trackers",       new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_enable_trackers, c)));
   variables->insert("encoding_list",         new utils::VariableStringSlot(rak::value_fn(std::string()), rak::bind_ptr_fn(&apply_encoding_list, c)));
+
+  variables->insert("encryption",            new utils::VariableStringSlot(rak::value_fn(std::string()), rak::ptr_fn(&apply_encryption)));
+  variables->insert("handshake_log",         new utils::VariableBool(false));
 }
