@@ -44,6 +44,7 @@
 #include <torrent/connection_manager.h>
 #include <torrent/torrent.h>
 #include <torrent/tracker_list.h>
+#include <torrent/data/file_list.h>
 
 #include "core/download.h"
 #include "input/manager.h"
@@ -95,7 +96,7 @@ Download::Download(core::Download* d) :
 
 Download::~Download() {
   if (is_active())
-    throw torrent::client_error("ui::Download::~Download() called on an active object.");
+    throw torrent::internal_error("ui::Download::~Download() called on an active object.");
 
   std::for_each(m_uiArray, m_uiArray + DISPLAY_MAX_SIZE, rak::call_delete<ElementBase>());
 
@@ -156,7 +157,7 @@ Download::create_info() {
   element->push_column("Tied to file:",     te_variable_string("tied_to_file"));
 
   element->push_back("");
-  element->push_column("Chunks:",           te_value(&torrent::Download::chunks_done), " / ", te_value(&torrent::Download::chunks_total), " * ", te_value(&torrent::Download::chunks_size));
+  element->push_column("Chunks:",           te_value(&torrent::FileList::completed_chunks), " / ", te_value(&torrent::FileList::size_chunks), " * ", te_value(&torrent::FileList::chunk_size));
   element->push_column("Priority:",         te_variable_value("priority"));
 
   element->push_column("State changed:",    te_variable_value("state_changed", value_base::flag_timer | value_base::flag_elapsed));
@@ -164,7 +165,7 @@ Download::create_info() {
   element->push_back("");
   element->push_column("Memory usage:",     te_value(&torrent::ChunkManager::memory_usage, value_base::flag_mb), " MB");
   element->push_column("Max memory usage:", te_value(&torrent::ChunkManager::max_memory_usage, value_base::flag_mb), " MB");
-  element->push_column("Free diskspace:",   te_value(&torrent::Download::free_diskspace, value_base::flag_mb), " MB");
+  element->push_column("Free diskspace:",   te_value(&torrent::FileList::free_diskspace, value_base::flag_mb), " MB");
   element->push_column("Safe diskspace:",   te_value(&torrent::ChunkManager::safe_free_diskspace, value_base::flag_mb), " MB");
 
   element->push_back("");
@@ -175,6 +176,8 @@ Download::create_info() {
   element->push_back("");
   element->push_column("Upload:",           te_variable_value("up_rate", value_base::flag_kb), " KB / ", te_variable_value("up_total", value_base::flag_xb));
   element->push_column("Download:",         te_variable_value("down_rate", value_base::flag_kb), " KB / ", te_variable_value("down_total", value_base::flag_xb));
+  element->push_column("Skipped:",          te_variable_value("skip_rate", value_base::flag_kb), " KB / ", te_variable_value("skip_total", value_base::flag_xb));
+  element->push_column("Preload:",          te_value(&torrent::ChunkManager::preload_type), " / ", te_value(&torrent::ChunkManager::stats_preloaded), " / ", te_value(&torrent::ChunkManager::stats_not_preloaded));
 
   element->set_column_width(element->column_width() + 1);
 
@@ -184,7 +187,7 @@ Download::create_info() {
 void
 Download::activate(display::Frame* frame, bool focus) {
   if (is_active())
-    throw torrent::client_error("ui::Download::activate() called on an already activated object.");
+    throw torrent::internal_error("ui::Download::activate() called on an already activated object.");
 
   control->input()->push_back(&m_bindings);
 
@@ -200,7 +203,7 @@ Download::activate(display::Frame* frame, bool focus) {
 void
 Download::disable() {
   if (!is_active())
-    throw torrent::client_error("ui::Download::disable() called on an already disabled object.");
+    throw torrent::internal_error("ui::Download::disable() called on an already disabled object.");
 
   control->input()->erase(&m_bindings);
 
@@ -215,10 +218,10 @@ Download::disable() {
 void
 Download::activate_display(Display displayType, bool focusDisplay) {
   if (!is_active())
-    throw torrent::client_error("ui::Download::activate_display(...) !is_active().");
+    throw torrent::internal_error("ui::Download::activate_display(...) !is_active().");
 
   if (displayType > DISPLAY_MAX_SIZE)
-    throw torrent::client_error("ui::Download::activate_display(...) out of bounds");
+    throw torrent::internal_error("ui::Download::activate_display(...) out of bounds");
 
   if (focusDisplay == m_focusDisplay && displayType == m_state)
     return;
